@@ -16,15 +16,21 @@
 #This program is distributed in the hope that it will be useful, but WITHOUT ANY
 #WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 #PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
+#
+# Changelog:
+# APS 01/29/2004: Changed run_nmap_scan to use parsescan().
+#		  $nmap_exe is set to default 'nmap' if find_exe returns empty
+#
+#
+#
 use strict;
 use Nmap::Parser::XML;
 use Getopt::Long;
 use File::Spec;
 use Pod::Usage;
 use vars qw(%G);
-use constant CMD1 => 'nmap -sS -O -v -v -v -oX - ';
-use constant CMD2 => 'nmap -sT -O -F -v -v -v -oX - ';
+use constant CMD1 => '-sV -O ';
+use constant CMD2 => '-sV -O -F';
 use constant TEST_FILE => 'example.xml';
 
 Getopt::Long::Configure('bundling');
@@ -101,7 +107,7 @@ sub port_service_print {
 
 #quick function to find an executable in a given path
 sub find_exe {
-shift if(ref($_[0]) eq caller());
+
 
     my $exe_to_find = shift;
     $exe_to_find =~ s/\.exe//;
@@ -129,7 +135,7 @@ shift if(ref($_[0]) eq caller());
 }
 
 sub run_nmap_scan {
-my @ips =  grep {/(?:\d+\.){3}\d+/} @_;
+my @ips =  @_;
 my $NMAP;
 	my $cmd;
 	if($G{fast}){
@@ -142,13 +148,13 @@ my $NMAP;
 
 	my $nmap_exe = find_exe('nmap');
 	if($nmap_exe eq '')
-	{print STDERR "ERROR: nmap executable not found in \$PATH\n";exit;}
+	{warn "ERROR: nmap executable not found in \$PATH\n";$nmap_exe = 'nmap';}
 
-	print 'Running: '.$cmd."\n" if($G{verbose} > 0);
+	print 'Running: '.$nmap_exe.' '.$cmd."\n" if($G{verbose} > 0);
+
+
+	$p->parsescan($nmap_exe,$cmd);
 
-	open $NMAP , "$cmd |" || die "ERROR: $!\n";
-	$p->parse($NMAP);
-	close $NMAP;
 return $p;
 }
 
@@ -199,6 +205,36 @@ the script will be.
 
 =back 4
 
+=head1 TARGET SPECIFICATION
+
+This documentation was taken from the nmap man page. The IP address inputs
+to this scripts should be in the nmap target specification format.
+
+The  simplest  case is listing single hostnames or IP addresses onthe command
+line. If you want to scan a subnet of  IP addresses, you can append '/mask' to
+the hostname or IP address. mask must be between 0 (scan the whole internet) and
+ 32 (scan the single host specified). Use /24 to scan a class 'C' address and
+ /16 for a class 'B'.
+
+You can use a more powerful notation which lets you specify an IP address
+using lists/ranges for each element. Thus you can scan the whole class 'B'
+network 128.210.*.* by specifying '128.210.*.*' or '128.210.0-255.0-255' or
+even use the mask notation: '128.210.0.0/16'. These are all equivalent.
+If you use asterisks ('*'), remember that most shells require you to escape
+them with  back  slashes or protect them with quotes.
+
+Another interesting thing to do is slice the Internet the other way.
+
+Examples:
+
+ scan_host.pl 127.0.0.1
+ scan_host.pl target.example.com
+ scan_host.pl target.example.com/24
+ scan_host.pl 10.210.*.1-127
+ scan_host.pl *.*.2.3-5
+ scan_host.pl 10.[10-15].10.[2-254]
+
+
 =head1 OUTPUT EXAMPLE
 
 These are ONLY examples of how the output would look like.
@@ -226,10 +262,9 @@ These are ONLY examples of how the output would look like.
 
 L<Nmap::Parser::XML>
 
-The Nmap::Parser::XML page can be found at:
-L<http://www.public.iastate.edu/~ironstar/Nmap-Parser-XML/>. It contains
-the latest developments on the module. The nmap security scanner homepage can
-be found at: L<http://www.insecure.org/nmap/>.
+The Nmap::Parser::XML page can be found at: L<http://npx.sourceforge.net/>.
+It contains the latest developments on the module. The nmap security scanner
+homepage can be found at: L<http://www.insecure.org/nmap/>.
 
 =head1 AUTHOR
 
